package javax.swing.text;

import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Reader;
import java.io.Writer;
import javax.swing.Action;

public class DefaultEditorKit extends EditorKit {
   public static final String EndOfLineStringProperty = "__EndOfLine__";
   public static final String insertContentAction = "insert-content";
   public static final String insertBreakAction = "insert-break";
   public static final String insertTabAction = "insert-tab";
   public static final String deletePrevCharAction = "delete-previous";
   public static final String deleteNextCharAction = "delete-next";
   public static final String readOnlyAction = "set-read-only";
   public static final String writableAction = "set-writable";
   public static final String cutAction = "cut-to-clipboard";
   public static final String copyAction = "copy-to-clipboard";
   public static final String pasteAction = "paste-from-clipboard";
   public static final String beepAction = "beep";
   public static final String pageUpAction = "page-up";
   public static final String pageDownAction = "page-down";
   static final String selectionPageUpAction = "selection-page-up";
   static final String selectionPageDownAction = "selection-page-down";
   public static final String forwardAction = "caret-forward";
   public static final String backwardAction = "caret-backward";
   public static final String selectionForwardAction = "selection-forward";
   public static final String selectionBackwardAction = "selection-backward";
   public static final String upAction = "caret-up";
   public static final String downAction = "caret-down";
   public static final String selectionUpAction = "selection-up";
   public static final String selectionDownAction = "selection-down";
   public static final String beginWordAction = "caret-begin-word";
   public static final String endWordAction = "caret-end-word";
   public static final String selectionBeginWordAction = "selection-begin-word";
   public static final String selectionEndWordAction = "selection-end-word";
   public static final String previousWordAction = "caret-previous-word";
   public static final String nextWordAction = "caret-next-word";
   public static final String selectionPreviousWordAction = "selection-previous-word";
   public static final String selectionNextWordAction = "selection-next-word";
   public static final String beginLineAction = "caret-begin-line";
   public static final String endLineAction = "caret-end-line";
   public static final String selectionBeginLineAction = "selection-begin-line";
   public static final String selectionEndLineAction = "selection-end-line";
   public static final String beginParagraphAction = "caret-begin-paragraph";
   public static final String endParagraphAction = "caret-end-paragraph";
   public static final String selectionBeginParagraphAction = "selection-begin-paragraph";
   public static final String selectionEndParagraphAction = "selection-end-paragraph";
   public static final String beginAction = "caret-begin";
   public static final String endAction = "caret-end";
   public static final String selectionBeginAction = "selection-begin";
   public static final String selectionEndAction = "selection-end";
   public static final String selectWordAction = "select-word";
   public static final String selectLineAction = "select-line";
   public static final String selectParagraphAction = "select-paragraph";
   public static final String selectAllAction = "select-all";
   static final String unselectAction = "unselect";
   public static final String defaultKeyTypedAction = "default-typed";
   private static final Action[] defaultActions = new Action[]{new InsertContentAction(), new DeletePrevCharAction(), new DeleteNextCharAction(), new ReadOnlyAction(), new WritableAction(), new CutAction(), new CopyAction(), new PasteAction(), new PageUpAction("page-up", false), new PageDownAction("page-down", false), new PageUpAction("selection-page-up", true), new PageDownAction("selection-page-down", true), new InsertBreakAction(), new BeepAction(), new NextVisualPositionAction("caret-forward", false, 3), new NextVisualPositionAction("caret-backward", false, 7), new NextVisualPositionAction("selection-forward", true, 3), new NextVisualPositionAction("selection-backward", true, 7), new NextVisualPositionAction("caret-up", false, 1), new NextVisualPositionAction("caret-down", false, 5), new NextVisualPositionAction("selection-up", true, 1), new NextVisualPositionAction("selection-down", true, 5), new BeginWordAction("caret-begin-word", false), new EndWordAction("caret-end-word", false), new BeginWordAction("selection-begin-word", true), new EndWordAction("selection-end-word", true), new PreviousWordAction("caret-previous-word", false), new NextWordAction("caret-next-word", false), new PreviousWordAction("selection-previous-word", true), new NextWordAction("selection-next-word", true), new BeginLineAction("caret-begin-line", false), new EndLineAction("caret-end-line", false), new BeginLineAction("selection-begin-line", true), new EndLineAction("selection-end-line", true), new BeginParagraphAction("caret-begin-paragraph", false), new EndParagraphAction("caret-end-paragraph", false), new BeginParagraphAction("selection-begin-paragraph", true), new EndParagraphAction("selection-end-paragraph", true), new BeginAction("caret-begin", false), new EndAction("caret-end", false), new BeginAction("selection-begin", true), new EndAction("selection-end", true), new DefaultKeyTypedAction(), new InsertTabAction(), new SelectWordAction(), new SelectLineAction(), new SelectParagraphAction(), new SelectAllAction(), new UnselectAction(), new DumpModelAction()};

   public Object clone() {
      return new DefaultEditorKit();
   }

   public Caret createCaret() {
      return null;
   }

   public Document createDefaultDocument() {
      return new PlainDocument();
   }

   public Action[] getActions() {
      return defaultActions;
   }

   public String getContentType() {
      return "text/plain";
   }

   public ViewFactory getViewFactory() {
      return null;
   }

   public void read(InputStream var1, Document var2, int var3) throws IOException, BadLocationException {
      this.read((Reader)(new InputStreamReader(var1)), var2, var3);
   }

   public void read(Reader var1, Document var2, int var3) throws IOException, BadLocationException {
      char[] var4 = new char[4096];
      boolean var6 = false;
      boolean var7 = false;
      boolean var8 = false;
      boolean var10 = var2.getLength() == 0;

      int var5;
      while((var5 = var1.read(var4, 0, var4.length)) != -1) {
         int var9 = 0;

         for(int var11 = 0; var11 < var5; ++var11) {
            switch (var4[var11]) {
               case '\n':
                  if (var6) {
                     if (var11 > var9 + 1) {
                        var2.insertString(var3, new String(var4, var9, var11 - var9 - 1), (AttributeSet)null);
                        var3 += var11 - var9 - 1;
                     }

                     var6 = false;
                     var9 = var11;
                     var7 = true;
                  }
                  break;
               case '\u000b':
               case '\f':
               default:
                  if (var6) {
                     var8 = true;
                     if (var11 == 0) {
                        var2.insertString(var3, "\n", (AttributeSet)null);
                        ++var3;
                     } else {
                        var4[var11 - 1] = '\n';
                     }

                     var6 = false;
                  }
                  break;
               case '\r':
                  if (var6) {
                     var8 = true;
                     if (var11 == 0) {
                        var2.insertString(var3, "\n", (AttributeSet)null);
                        ++var3;
                     } else {
                        var4[var11 - 1] = '\n';
                     }
                  } else {
                     var6 = true;
                  }
            }
         }

         if (var9 < var5) {
            if (var6) {
               if (var9 < var5 - 1) {
                  var2.insertString(var3, new String(var4, var9, var5 - var9 - 1), (AttributeSet)null);
                  var3 += var5 - var9 - 1;
               }
            } else {
               var2.insertString(var3, new String(var4, var9, var5 - var9), (AttributeSet)null);
               var3 += var5 - var9;
            }
         }
      }

      if (var6) {
         var2.insertString(var3, "\n", (AttributeSet)null);
         var8 = true;
      }

      if (var10) {
         if (var7) {
            var2.putProperty("__EndOfLine__", "\r\n");
         } else if (var8) {
            var2.putProperty("__EndOfLine__", "\r");
         } else {
            var2.putProperty("__EndOfLine__", "\n");
         }
      }

   }

   public void write(OutputStream var1, Document var2, int var3, int var4) throws IOException, BadLocationException {
      this.write((Writer)(new OutputStreamWriter(var1)), var2, var3, var4);
   }

   public void write(Writer var1, Document var2, int var3, int var4) throws IOException, BadLocationException {
      if (var3 >= 0 && var3 + var4 <= var2.getLength()) {
         Segment var5 = new Segment();
         int var6 = var4;
         int var7 = var3;
         Object var8 = var2.getProperty("__EndOfLine__");
         if (var8 == null) {
            try {
               var8 = System.getProperty("line.separator");
            } catch (SecurityException var15) {
            }
         }

         String var9;
         if (var8 instanceof String) {
            var9 = (String)var8;
         } else {
            var9 = null;
         }

         if (var8 != null && !var9.equals("\n")) {
            while(var6 > 0) {
               int var16 = Math.min(var6, 4096);
               var2.getText(var7, var16, var5);
               int var11 = var5.offset;
               char[] var12 = var5.array;
               int var13 = var11 + var5.count;

               for(int var14 = var11; var14 < var13; ++var14) {
                  if (var12[var14] == '\n') {
                     if (var14 > var11) {
                        var1.write(var12, var11, var14 - var11);
                     }

                     var1.write(var9);
                     var11 = var14 + 1;
                  }
               }

               if (var13 > var11) {
                  var1.write(var12, var11, var13 - var11);
               }

               var7 += var16;
               var6 -= var16;
            }
         } else {
            while(var6 > 0) {
               int var10 = Math.min(var6, 4096);
               var2.getText(var7, var10, var5);
               var1.write(var5.array, var5.offset, var5.count);
               var7 += var10;
               var6 -= var10;
            }
         }

         var1.flush();
      } else {
         throw new BadLocationException("DefaultEditorKit.write", var3);
      }
   }
}
