package javax.swing.text;

import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JEditorPane;

public class StyledEditorKit extends DefaultEditorKit {
   private static final ViewFactory defaultFactory = new StyledViewFactory();
   Element currentRun;
   Element currentParagraph;
   MutableAttributeSet inputAttributes = new 1(this);
   private AttributeTracker inputAttributeUpdater = new AttributeTracker(this);
   private static final Action[] defaultActions = new Action[]{new FontFamilyAction("font-family-SansSerif", "SansSerif"), new FontFamilyAction("font-family-Monospaced", "Monospaced"), new FontFamilyAction("font-family-Serif", "Serif"), new FontSizeAction("font-size-8", 8), new FontSizeAction("font-size-10", 10), new FontSizeAction("font-size-12", 12), new FontSizeAction("font-size-14", 14), new FontSizeAction("font-size-16", 16), new FontSizeAction("font-size-18", 18), new FontSizeAction("font-size-24", 24), new FontSizeAction("font-size-36", 36), new FontSizeAction("font-size-48", 48), new AlignmentAction("left-justify", 0), new AlignmentAction("center-justify", 1), new AlignmentAction("right-justify", 2), new BoldAction(), new ItalicAction(), new UnderlineAction()};

   public Object clone() {
      return new StyledEditorKit();
   }

   public Document createDefaultDocument() {
      return new DefaultStyledDocument();
   }

   protected void createInputAttributes(Element var1, MutableAttributeSet var2) {
      var2.removeAttributes(var2);
      var2.addAttributes(var1.getAttributes());
      var2.removeAttribute(StyleConstants.ComponentAttribute);
      var2.removeAttribute(StyleConstants.IconAttribute);
      var2.removeAttribute("$ename");
      var2.removeAttribute(StyleConstants.ComposedTextAttribute);
   }

   public void deinstall(JEditorPane var1) {
      ((JTextComponent)var1).removeCaretListener(this.inputAttributeUpdater);
      ((JComponent)var1).removePropertyChangeListener(this.inputAttributeUpdater);
      this.currentRun = null;
      this.currentParagraph = null;
   }

   public Action[] getActions() {
      return TextAction.augmentList(super.getActions(), defaultActions);
   }

   public Element getCharacterAttributeRun() {
      return this.currentRun;
   }

   public MutableAttributeSet getInputAttributes() {
      return this.inputAttributes;
   }

   public ViewFactory getViewFactory() {
      return defaultFactory;
   }

   public void install(JEditorPane var1) {
      ((JTextComponent)var1).addCaretListener(this.inputAttributeUpdater);
      ((JComponent)var1).addPropertyChangeListener(this.inputAttributeUpdater);
      Caret var2 = ((JTextComponent)var1).getCaret();
      if (var2 != null) {
         this.inputAttributeUpdater.updateInputAttributes(var2.getDot(), var2.getMark(), var1);
      }

   }
}
