package javax.swing.text;

import java.io.Serializable;

public class TabSet implements Serializable {
   private TabStop[] tabs;

   public TabSet(TabStop[] var1) {
      if (var1 != null) {
         int var2 = var1.length;
         this.tabs = new TabStop[var2];
         System.arraycopy(var1, 0, this.tabs, 0, var2);
      } else {
         this.tabs = null;
      }

   }

   public TabStop getTab(int var1) {
      int var2 = this.getTabCount();
      if (var1 >= 0 && var1 < var2) {
         return this.tabs[var1];
      } else {
         throw new IllegalArgumentException(var1 + " is outside the range of tabs");
      }
   }

   public TabStop getTabAfter(float var1) {
      int var2 = this.getTabIndexAfter(var1);
      return var2 == -1 ? null : this.tabs[var2];
   }

   public int getTabCount() {
      return this.tabs == null ? 0 : this.tabs.length;
   }

   public int getTabIndex(TabStop var1) {
      for(int var2 = this.getTabCount() - 1; var2 >= 0; --var2) {
         if (this.getTab(var2) == var1) {
            return var2;
         }
      }

      return -1;
   }

   public int getTabIndexAfter(float var1) {
      int var3 = 0;
      int var4 = this.getTabCount();

      while(var3 != var4) {
         int var2 = (var4 - var3) / 2 + var3;
         if (!(var1 > this.tabs[var2].getPosition())) {
            if (var2 == 0 || var1 > this.tabs[var2 - 1].getPosition()) {
               return var2;
            }

            var4 = var2;
         } else if (var3 == var2) {
            var3 = var4;
         } else {
            var3 = var2;
         }
      }

      return -1;
   }

   public String toString() {
      int var1 = this.getTabCount();
      StringBuffer var2 = new StringBuffer("[ ");

      for(int var3 = 0; var3 < var1; ++var3) {
         if (var3 > 0) {
            var2.append(" - ");
         }

         var2.append(this.getTab(var3).toString());
      }

      var2.append(" ]");
      return var2.toString();
   }
}
