package javax.swing.text;

import java.awt.Container;
import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.Shape;
import javax.swing.SwingConstants;
import javax.swing.event.DocumentEvent;
import javax.swing.text.Position.Bias;

public abstract class View implements SwingConstants {
   public static final int BadBreakWeight = 0;
   public static final int GoodBreakWeight = 1000;
   public static final int ExcellentBreakWeight = 2000;
   public static final int ForcedBreakWeight = 3000;
   public static final int X_AXIS = 0;
   public static final int Y_AXIS = 1;
   static final Position.Bias[] sharedBiasReturn = new Position.Bias[1];
   private View parent;
   private Element elem;

   public View(Element var1) {
      this.elem = var1;
   }

   public View breakView(int var1, int var2, float var3, float var4) {
      return this;
   }

   public void changedUpdate(DocumentEvent var1, Shape var2, ViewFactory var3) {
   }

   public View createFragment(int var1, int var2) {
      return this;
   }

   public float getAlignment(int var1) {
      return 0.5F;
   }

   public AttributeSet getAttributes() {
      return this.elem.getAttributes();
   }

   public int getBreakWeight(int var1, float var2, float var3) {
      return var3 > this.getPreferredSpan(var1) ? 1000 : 0;
   }

   public Shape getChildAllocation(int var1, Shape var2) {
      return null;
   }

   public Container getContainer() {
      View var1 = this.getParent();
      return var1 != null ? var1.getContainer() : null;
   }

   public Document getDocument() {
      return this.elem.getDocument();
   }

   public Element getElement() {
      return this.elem;
   }

   public int getEndOffset() {
      return this.elem.getEndOffset();
   }

   public float getMaximumSpan(int var1) {
      int var2 = this.getResizeWeight(var1);
      return var2 == 0 ? this.getPreferredSpan(var1) : (float)Integer.MAX_VALUE;
   }

   public float getMinimumSpan(int var1) {
      int var2 = this.getResizeWeight(var1);
      return var2 == 0 ? this.getPreferredSpan(var1) : 0.0F;
   }

   public int getNextVisualPositionFrom(int var1, Position.Bias var2, Shape var3, int var4, Position.Bias[] var5) throws BadLocationException {
      var5[0] = Bias.Forward;
      switch (var4) {
         case 1:
            JTextComponent var9 = (JTextComponent)this.getContainer();
            Rectangle var10 = var9.modelToView(var1);
            var1 = Utilities.getPositionAbove(var9, var1, var10.x);
            break;
         case 2:
         case 4:
         case 6:
         default:
            throw new IllegalArgumentException("Bad direction: " + var4);
         case 3:
            if (var1 == -1) {
               var1 = this.getStartOffset();
            } else {
               var1 = Math.min(var1 + 1, this.getDocument().getLength());
            }
            break;
         case 5:
            JTextComponent var6 = (JTextComponent)this.getContainer();
            Rectangle var7 = var6.modelToView(var1);
            var1 = Utilities.getPositionBelow(var6, var1, var7.x);
            break;
         case 7:
            if (var1 == -1) {
               var1 = Math.max(0, this.getEndOffset() - 1);
            } else {
               var1 = Math.max(0, var1 - 1);
            }
      }

      return var1;
   }

   public View getParent() {
      return this.parent;
   }

   public abstract float getPreferredSpan(int var1);

   public int getResizeWeight(int var1) {
      return 0;
   }

   public int getStartOffset() {
      return this.elem.getStartOffset();
   }

   public View getView(int var1) {
      return null;
   }

   public int getViewCount() {
      return 0;
   }

   public ViewFactory getViewFactory() {
      View var1 = this.getParent();
      return var1 != null ? var1.getViewFactory() : null;
   }

   public void insertUpdate(DocumentEvent var1, Shape var2, ViewFactory var3) {
   }

   public boolean isVisible() {
      return true;
   }

   /** @deprecated */
   public Shape modelToView(int var1, Shape var2) throws BadLocationException {
      return this.modelToView(var1, var2, Bias.Forward);
   }

   public abstract Shape modelToView(int var1, Shape var2, Position.Bias var3) throws BadLocationException;

   public Shape modelToView(int var1, Position.Bias var2, int var3, Position.Bias var4, Shape var5) throws BadLocationException {
      Shape var6 = this.modelToView(var1, var5, var2);
      Object var7;
      if (var3 == this.getEndOffset()) {
         try {
            var7 = this.modelToView(var3, var5, var4);
         } catch (BadLocationException var11) {
            var7 = null;
         }

         if (var7 == null) {
            Rectangle var8 = var5 instanceof Rectangle ? (Rectangle)var5 : var5.getBounds();
            var7 = new Rectangle(var8.x + var8.width - 1, var8.y, 1, var8.height);
         }
      } else {
         var7 = this.modelToView(var3, var5, var4);
      }

      Rectangle var12 = var6.getBounds();
      Rectangle var9 = var7 instanceof Rectangle ? (Rectangle)var7 : ((Shape)var7).getBounds();
      if (var12.y != var9.y) {
         Rectangle var10 = var5 instanceof Rectangle ? (Rectangle)var5 : var5.getBounds();
         var12.x = var10.x;
         var12.width = var10.width;
      }

      var12.add(var9);
      return var12;
   }

   public abstract void paint(Graphics var1, Shape var2);

   public void preferenceChanged(View var1, boolean var2, boolean var3) {
      View var4 = this.getParent();
      if (var4 != null) {
         var4.preferenceChanged(this, var2, var3);
      }

   }

   public void removeUpdate(DocumentEvent var1, Shape var2, ViewFactory var3) {
   }

   public void setParent(View var1) {
      this.parent = var1;
   }

   public void setSize(float var1, float var2) {
   }

   /** @deprecated */
   public int viewToModel(float var1, float var2, Shape var3) {
      sharedBiasReturn[0] = Bias.Forward;
      return this.viewToModel(var1, var2, var3, sharedBiasReturn);
   }

   public abstract int viewToModel(float var1, float var2, Shape var3, Position.Bias[] var4);
}
