package javax.swing.text.html;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Container;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Shape;
import java.awt.Toolkit;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.Border;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.AttributeSet;
import javax.swing.text.ComponentView;
import javax.swing.text.Document;
import javax.swing.text.Element;
import javax.swing.text.JTextComponent;
import javax.swing.text.SimpleAttributeSet;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledDocument;
import javax.swing.text.View;
import javax.swing.text.ViewFactory;
import javax.swing.text.html.HTML.Attribute;

class HiddenTagView extends EditableView implements DocumentListener {
   float yAlign = 1.0F;
   boolean isSettingAttributes;
   static final int circleR = 3;
   static final int circleD = 6;
   static final int tagSize = 6;
   static final int padding = 3;
   static final Color UnknownTagBorderColor;
   static final Border StartBorder;
   static final Border EndBorder;

   static {
      UnknownTagBorderColor = Color.black;
      StartBorder = new StartTagBorder();
      EndBorder = new EndTagBorder();
   }

   HiddenTagView(Element var1) {
      super(var1);
   }

   public void changedUpdate(DocumentEvent var1) {
      this.pushTextToModel();
   }

   public void changedUpdate(DocumentEvent var1, Shape var2, ViewFactory var3) {
      if (!this.isSettingAttributes) {
         this.isSettingAttributes = true;

         try {
            this.getTextComponent().setText(this.getRepresentedText());
            this.resetBorder();
         } finally {
            this.isSettingAttributes = false;
         }

         ((View)this).preferenceChanged(this, true, true);
         ((View)this).getContainer().repaint();
      }

   }

   protected Component createComponent() {
      JTextField var1 = new JTextField(((View)this).getElement().getName());
      Document var2 = ((View)this).getDocument();
      Font var3;
      if (var2 instanceof StyledDocument) {
         var3 = ((StyledDocument)var2).getFont(((View)this).getAttributes());
         var1.setFont(var3);
      } else {
         var3 = ((Component)var1).getFont();
      }

      ((JTextComponent)var1).getDocument().addDocumentListener(this);
      this.updateYAlign(var3);
      JPanel var4 = new JPanel(new BorderLayout());
      ((JComponent)var4).setBackground((Color)null);
      if (this.isEndTag()) {
         ((JComponent)var4).setBorder(EndBorder);
      } else {
         ((JComponent)var4).setBorder(StartBorder);
      }

      ((Container)var4).add(var1);
      return var4;
   }

   public float getAlignment(int var1) {
      return var1 == 1 ? this.yAlign : 0.5F;
   }

   public float getMaximumSpan(int var1) {
      return var1 == 0 && ((EditableView)this).isVisible() ? Math.max(30.0F, super.getMaximumSpan(var1)) : super.getMaximumSpan(var1);
   }

   public float getMinimumSpan(int var1) {
      return var1 == 0 && ((EditableView)this).isVisible() ? Math.max(30.0F, super.getPreferredSpan(var1)) : super.getMinimumSpan(var1);
   }

   public float getPreferredSpan(int var1) {
      return var1 == 0 && ((EditableView)this).isVisible() ? Math.max(30.0F, super.getPreferredSpan(var1)) : super.getPreferredSpan(var1);
   }

   String getRepresentedText() {
      String var1 = ((View)this).getElement().getName();
      return var1 == null ? "" : var1;
   }

   JTextComponent getTextComponent() {
      return (JTextComponent)((Container)((ComponentView)this).getComponent()).getComponent(0);
   }

   public void insertUpdate(DocumentEvent var1) {
      this.pushTextToModel();
   }

   boolean isEndTag() {
      AttributeSet var1 = ((View)this).getElement().getAttributes();
      if (var1 != null) {
         Object var2 = var1.getAttribute(Attribute.ENDTAG);
         if (var2 != null && var2 instanceof String && ((String)var2).equals("true")) {
            return true;
         }
      }

      return false;
   }

   void pushTextToModel() {
      if (!this.isSettingAttributes) {
         Object var1 = ((View)this).getElement().getAttributes().getAttribute(StyleConstants.NameAttribute);
         Document var2 = ((View)this).getDocument();
         if (var1 instanceof HTML.UnknownTag && var2 instanceof StyledDocument) {
            SimpleAttributeSet var3 = new SimpleAttributeSet();
            String var4 = this.getTextComponent().getText();
            this.isSettingAttributes = true;

            try {
               var3.addAttribute(StyleConstants.NameAttribute, new HTML.UnknownTag(var4));
               ((StyledDocument)var2).setCharacterAttributes(((View)this).getStartOffset(), ((View)this).getEndOffset() - ((View)this).getStartOffset(), var3, false);
            } finally {
               this.isSettingAttributes = false;
            }
         }
      }

   }

   public void removeUpdate(DocumentEvent var1) {
      this.pushTextToModel();
   }

   void resetBorder() {
      if (this.isEndTag()) {
         ((JPanel)((ComponentView)this).getComponent()).setBorder(EndBorder);
      } else {
         ((JPanel)((ComponentView)this).getComponent()).setBorder(StartBorder);
      }

   }

   void updateYAlign(Font var1) {
      FontMetrics var2 = Toolkit.getDefaultToolkit().getFontMetrics(var1);
      float var3 = (float)var2.getHeight();
      float var4 = (float)var2.getDescent();
      float var5 = (var3 - var4) / var3;
   }
}
