package javax.swing.text.html;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.io.Serializable;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.StringTokenizer;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.text.AttributeSet;
import javax.swing.text.StyleConstants;
import javax.swing.text.StyledDocument;
import javax.swing.text.View;
import javax.swing.text.html.CSS.Attribute;
import javax.swing.text.html.CSS.Value;
import javax.swing.text.html.HTML.Tag;

public class StyleSheet$ListPainter implements Serializable {
   static final char[][] romanChars = new char[][]{{'i', 'v'}, {'x', 'l'}, {'c', 'd'}, {'m', '?'}};
   private int start;
   private CSS.Value type;
   URL imageurl;
   Icon img = null;
   private int bulletgap = 5;

   StyleSheet$ListPainter(AttributeSet var1) {
      String var2 = (String)var1.getAttribute(Attribute.LIST_STYLE_IMAGE);
      this.type = null;
      if (var2 != null && !var2.equals("none")) {
         try {
            String var3 = null;
            StringTokenizer var4 = new StringTokenizer(var2, "()");
            if (var4.hasMoreTokens()) {
               var3 = var4.nextToken();
            }

            if (var4.hasMoreTokens()) {
               var3 = var4.nextToken();
            }

            URL var5 = new URL(var3);
            this.img = new ImageIcon(var5);
         } catch (MalformedURLException var6) {
            this.type = null;
            this.img = null;
         }
      }

      if (this.img == null) {
         this.type = (CSS.Value)var1.getAttribute(Attribute.LIST_STYLE_TYPE);
      }

      this.start = 1;
   }

   void drawIcon(Graphics var1, int var2, int var3, int var4, float var5, Component var6) {
      var1.setColor(Color.black);
      int var7 = var2 - this.img.getIconWidth() - this.bulletgap;
      int var8 = var3 + (int)((float)var4 * var5) - 3;
      this.img.paintIcon(var6, var1, var7, var8);
   }

   void drawLetter(Graphics var1, char var2, int var3, int var4, int var5, int var6) {
      var1.setColor(Color.black);
      String var7 = this.formatItemNum(var6, var2) + ".";
      FontMetrics var8 = var1.getFontMetrics();
      int var9 = var8.stringWidth(var7);
      int var10 = var3 - var9 - this.bulletgap;
      int var11 = var4 + var8.getAscent() + var8.getLeading();
      var1.drawString(var7, var10, var11);
   }

   void drawShape(Graphics var1, CSS.Value var2, int var3, int var4, int var5, float var6) {
      var1.setColor(Color.black);
      int var7 = var3 - this.bulletgap - 7;
      int var8 = var4 + (int)((float)var5 * var6) - 3;
      if (var2 == Value.SQUARE) {
         var1.drawRect(var7, var8, 7, 7);
      } else if (var2 == Value.CIRCLE) {
         var1.drawOval(var7, var8, 7, 7);
      } else {
         var1.fillOval(var7, var8, 7, 7);
      }

   }

   String formatAlphaNumerals(int var1) {
      String var2 = "";
      if (var1 > 26) {
         var2 = this.formatAlphaNumerals(var1 / 26) + this.formatAlphaNumerals(var1 % 26);
      } else {
         var2 = String.valueOf((char)(97 + var1 - 1));
      }

      return var2;
   }

   String formatItemNum(int var1, char var2) {
      String var3 = "1";
      boolean var4 = false;
      String var5;
      switch (var2) {
         case '1':
         default:
            var5 = String.valueOf(var1);
            break;
         case 'A':
            var4 = true;
         case 'a':
            var5 = this.formatAlphaNumerals(var1);
            break;
         case 'I':
            var4 = true;
         case 'i':
            var5 = this.formatRomanNumerals(var1);
      }

      if (var4) {
         var5 = var5.toUpperCase();
      }

      return var5;
   }

   String formatRomanDigit(int var1, int var2) {
      String var3 = "";
      if (var2 == 9) {
         var3 = var3 + romanChars[var1][0];
         var3 = var3 + romanChars[var1 + 1][0];
         return var3;
      } else if (var2 == 4) {
         var3 = var3 + romanChars[var1][0];
         var3 = var3 + romanChars[var1][1];
         return var3;
      } else {
         if (var2 >= 5) {
            var3 = var3 + romanChars[var1][1];
            var2 -= 5;
         }

         for(int var4 = 0; var4 < var2; ++var4) {
            var3 = var3 + romanChars[var1][0];
         }

         return var3;
      }
   }

   String formatRomanNumerals(int var1) {
      return this.formatRomanNumerals(0, var1);
   }

   String formatRomanNumerals(int var1, int var2) {
      return var2 < 10 ? this.formatRomanDigit(var1, var2) : this.formatRomanNumerals(var1 + 1, var2 / 10) + this.formatRomanDigit(var1, var2 % 10);
   }

   private CSS.Value getChildType(View var1) {
      CSS.Value var2 = (CSS.Value)var1.getAttributes().getAttribute(Attribute.LIST_STYLE_TYPE);
      if (var2 == null) {
         if (this.type == null) {
            View var3 = var1.getParent();
            HTMLDocument var4 = (HTMLDocument)var3.getDocument();
            if (var4.matchNameAttribute(var3.getElement().getAttributes(), Tag.OL)) {
               var2 = Value.DECIMAL;
            } else {
               var2 = Value.DISC;
            }
         } else {
            var2 = this.type;
         }
      }

      return var2;
   }

   private int getRenderIndex(View var1, int var2) {
      int var3 = var2;

      for(int var4 = var2; var4 >= 0; --var4) {
         AttributeSet var5 = var1.getElement().getElement(var4).getAttributes();
         if (var5.getAttribute(StyleConstants.NameAttribute) != Tag.LI) {
            --var3;
         } else {
            Object var6 = var5.getAttribute(javax.swing.text.html.HTML.Attribute.VALUE);
            if (var6 != null && var6 instanceof String) {
               try {
                  int var7 = Integer.parseInt((String)var6);
                  return var3 - var4 + var7;
               } catch (NumberFormatException var8) {
               }
            }
         }
      }

      return var3 + this.start;
   }

   public void paint(Graphics var1, float var2, float var3, float var4, float var5, View var6, int var7) {
      View var8 = var6.getView(var7);
      Object var9 = var8.getElement().getAttributes().getAttribute(StyleConstants.NameAttribute);
      if (var9 instanceof HTML.Tag && var9 == Tag.LI) {
         CSS.Value var10 = this.getChildType(var8);
         float var11 = var8.getAlignment(1);
         Font var12 = ((StyledDocument)var8.getDocument()).getFont(var8.getAttributes());
         if (var12 != null) {
            var1.setFont(var12);
         }

         if (var10 != Value.SQUARE && var10 != Value.CIRCLE && var10 != Value.DISC) {
            if (var10 == Value.CIRCLE) {
               this.drawShape(var1, var10, (int)var2, (int)var3, (int)var5, var11);
            } else if (var10 == Value.DECIMAL) {
               this.drawLetter(var1, '1', (int)var2, (int)var3, (int)var5, this.getRenderIndex(var6, var7));
            } else if (var10 == Value.LOWER_ALPHA) {
               this.drawLetter(var1, 'a', (int)var2, (int)var3, (int)var5, this.getRenderIndex(var6, var7));
            } else if (var10 == Value.UPPER_ALPHA) {
               this.drawLetter(var1, 'A', (int)var2, (int)var3, (int)var5, this.getRenderIndex(var6, var7));
            } else if (var10 == Value.LOWER_ROMAN) {
               this.drawLetter(var1, 'i', (int)var2, (int)var3, (int)var5, this.getRenderIndex(var6, var7));
            } else if (var10 == Value.UPPER_ROMAN) {
               this.drawLetter(var1, 'I', (int)var2, (int)var3, (int)var5, this.getRenderIndex(var6, var7));
            } else if (var10 == null && this.img != null) {
               this.drawIcon(var1, (int)var2, (int)var3, (int)var5, var11, var6.getContainer());
            }
         } else {
            this.drawShape(var1, var10, (int)var2, (int)var3, (int)var5, var11);
         }

      }
   }
}
