package javax.swing.text.html;

import java.awt.Graphics;
import java.awt.Rectangle;
import java.awt.Shape;
import javax.swing.SizeRequirements;
import javax.swing.event.DocumentEvent;
import javax.swing.text.AttributeSet;
import javax.swing.text.BoxView;
import javax.swing.text.CompositeView;
import javax.swing.text.Element;
import javax.swing.text.StyleConstants;
import javax.swing.text.View;
import javax.swing.text.ViewFactory;
import javax.swing.text.html.CSS.Attribute;
import javax.swing.text.html.HTML.Tag;

class TableView extends javax.swing.text.TableView {
   private AttributeSet attr;
   private StyleSheet.BoxPainter painter;

   public TableView(Element var1) {
      super(var1);
      StyleSheet var2 = this.getStyleSheet();
      this.attr = var2.getViewAttributes(this);
   }

   void adjustSizeForCSS(int var1, SizeRequirements var2) {
      if (var1 == 0) {
         Object var3 = this.attr.getAttribute(Attribute.WIDTH);
         if (var3 != null) {
            int var4 = (int)((CSS.LengthValue)var3).getValue();
            var2.minimum = var2.preferred = var4;
            var2.maximum = Math.max(var2.maximum, var4);
         }
      } else {
         Object var5 = this.attr.getAttribute(Attribute.HEIGHT);
         if (var5 != null) {
            int var6 = (int)((CSS.LengthValue)var5).getValue();
            var2.minimum = var2.preferred = var6;
            var2.maximum = Math.max(var2.maximum, var6);
         }
      }

   }

   protected SizeRequirements calculateMinorAxisRequirements(int var1, SizeRequirements var2) {
      SizeRequirements var3 = super.calculateMinorAxisRequirements(var1, var2);
      this.adjustSizeForCSS(var1, var3);
      return var3;
   }

   public void changedUpdate(DocumentEvent var1, Shape var2, ViewFactory var3) {
      super.changedUpdate(var1, var2, this.createViewFactory(var3));
   }

   protected javax.swing.text.TableView.TableRow createTableRow(Element var1) {
      Object var2 = var1.getAttributes().getAttribute(StyleConstants.NameAttribute);
      return var2 == Tag.TR ? new RowView(this, var1) : null;
   }

   ViewFactory createViewFactory(ViewFactory var1) {
      return new TableFactory(this, var1);
   }

   public AttributeSet getAttributes() {
      return this.attr;
   }

   public float getMaximumSpan(int var1) {
      return ((BoxView)this).getPreferredSpan(var1);
   }

   protected StyleSheet getStyleSheet() {
      HTMLDocument var1 = (HTMLDocument)((View)this).getDocument();
      return var1.getStyleSheet();
   }

   public ViewFactory getViewFactory() {
      return this.createViewFactory(super.getViewFactory());
   }

   public void insertUpdate(DocumentEvent var1, Shape var2, ViewFactory var3) {
      super.insertUpdate(var1, var2, this.createViewFactory(var3));
   }

   public void paint(Graphics var1, Shape var2) {
      Rectangle var3 = (Rectangle)var2;
      this.painter.paint(var1, (float)var3.x, (float)var3.y, (float)var3.width, (float)var3.height, this);
      super.paint(var1, var3);
   }

   public void removeUpdate(DocumentEvent var1, Shape var2, ViewFactory var3) {
      super.removeUpdate(var1, var2, this.createViewFactory(var3));
   }

   public void setParent(View var1) {
      super.setParent(var1);
      StyleSheet var2 = this.getStyleSheet();
      this.painter = var2.getBoxPainter(this.attr);
      this.setPropertiesFromAttributes();
   }

   protected void setPropertiesFromAttributes() {
      if (this.attr != null) {
         ((CompositeView)this).setInsets((short)((int)this.painter.getInset(1, this)), (short)((int)this.painter.getInset(2, this)), (short)((int)this.painter.getInset(3, this)), (short)((int)this.painter.getInset(4, this)));
      }

   }
}
